@@ -2,4 +2,7 @@
 pub mod compression;
 pub mod general;
 pub mod dates;
-pub mod rapidlogging;
\ No newline at end of file
+pub mod rapidlogging;
+pub mod unit_conversion;
+pub mod colours;
+pub mod stringhelpers;
\ No newline at end of file
