@@ -1,7 +1,7 @@
 //! General purpose functionalities for computer science. Got any wishes? Tell us on GitHub or our Discord.
 use crate::math::general::NumTools;
-use std::{intrinsics::transmute, fmt::Debug, ops::{BitAndAssign, Shl, BitOrAssign, BitXorAssign}, convert::TryFrom};
-use core::mem::{size_of_val, size_of};
+use std::{intrinsics::transmute, fmt::Debug, ops::{*}, convert::TryFrom};
+use core::mem::*;
 
 const BITWISE_ERR1: &str = "Arguments were not the same size in memory.";
 const BITWISE_ERR2: &str = "U is bigger than T. Consider reversing the arguments.";
@@ -81,7 +81,7 @@ pub trait BitwiseSlice<T, U> {
     /// assert_eq!(vec!(0, 0), fs.xor_with(&sn));
     /// assert_eq!(vec!(259), fs2.xor_with(&sn2));
     /// ```
-    #[must_use]
+    #[must_use = "This returns the result of the operation, without modifying the original."]
     fn xor_with(&self, other: &[U]) -> Vec<T>;
     /// ORs a slice of type `[T]` with a slice of type `[U]`.
     /// # Returns
@@ -101,7 +101,7 @@ pub trait BitwiseSlice<T, U> {
     /// assert_eq!(vec!(129, 2), fs.or_with(&sn));
     /// assert_eq!(vec!(7), fs2.or_with(&sn2));
     /// ```
-    #[must_use]
+    #[must_use = "This returns the result of the operation, without modifying the original."]
     fn or_with(&self, other: &[U]) -> Vec<T>;
     /// ANDs a slice of type `[T]` with a slice of type `[U]`.
     /// # Returns
@@ -121,19 +121,16 @@ pub trait BitwiseSlice<T, U> {
     /// assert_eq!(vec!(1, 2), fs.and_with(&sn));
     /// assert_eq!(vec!(1), fs2.and_with(&sn2));
     /// ```
-    #[must_use]
+    #[must_use = "This returns the result of the operation, without modifying the original."]
     fn and_with(&self, other: &[U]) -> Vec<T>;
 }
 /// Trait for `binary_insert`.
 pub trait BinaryInsert<T> {
     /// Insert an element into a ***sorted*** `vec` whilst maintaining the order, consuming `other`.
-    ///
     /// # Arguments
     /// * `value` - The value which needs to be inserted.
-    ///
     /// # Returns
     /// Nothing.
-    ///
     /// # Warning
     /// This function ***will not*** check if the parsed `vec` is sorted.
     /// # Examples
@@ -174,108 +171,6 @@ pub trait BinaryInsert<T> {
     /// ```
     fn binary_insert_no_dup(&mut self, value: T);
 }
-/// Trait for a kind of indexing Strings in Rust.
-pub trait StringIndex {
-    /// Get the `char` at a given index from a `String` or `&str`.
-    /// If only dealing with ASCII-characters, `byte_at` is recommended.
-    /// 
-    /// # Arguments
-    /// * `index` - The index of the character
-    ///
-    /// # Returns
-    /// A `Option<char>`.
-    /// 
-    /// # Examples
-    /// ```
-    /// use lib_rapid::compsci::general::StringIndex;
-    /// 
-    /// let s = String::from("Hello");
-    /// assert_eq!('H', s.char_at(0).unwrap());
-    /// assert_eq!('e', s.char_at(1).unwrap());
-    /// assert_eq!('l', s.char_at(2).unwrap());
-    /// ```
-    /// ```
-    /// use lib_rapid::compsci::general::StringIndex;
-    /// 
-    /// let s = "Hello";
-    /// assert_eq!('H', s.char_at(0).unwrap());
-    /// assert_eq!('e', s.char_at(1).unwrap());
-    /// assert_eq!('l', s.char_at(2).unwrap());
-    /// ```
-    #[must_use]
-    fn char_at(&self, index: usize) -> Option<char>;
-    /// Get the byte at a given index from a `String` or `&str`.
-    /// If dealing with Unicode characters, this function is *not* recommended, as it only returns one byte even though Unicode can be a multi byte encoding.
-    /// 
-    /// # Arguments
-    /// * `index` - The index of the byte
-    ///
-    /// # Returns
-    /// A `u8`.
-    /// 
-    /// # Examples
-    /// ```
-    /// use lib_rapid::compsci::general::StringIndex;
-    /// 
-    /// let s = String::from("Hello");
-    /// assert_eq!('H', s.byte_at(0) as char);
-    /// assert_eq!('e', s.byte_at(1) as char);
-    /// assert_eq!('l', s.byte_at(2) as char);
-    /// ```
-    /// ```
-    /// use lib_rapid::compsci::general::StringIndex;
-    /// 
-    /// let s = "Hello";
-    /// assert_eq!('H', s.byte_at(0) as char);
-    /// assert_eq!('e', s.byte_at(1) as char);
-    /// assert_eq!('l', s.byte_at(2) as char);
-    /// ```
-    #[must_use]
-    fn byte_at(&self, index: usize) -> u8;
-}
-/// Trait for functions related to brackets.
-pub trait Brackets {
-    /// Determines whether the given `&str` or `String` has valid brackets.
-    /// # Returns
-    /// `Ok(true)` if all brackets were closed, otherwise `Err(usize)`, where `usize` is the index of the String at which the error occured.
-    /// # Examples
-    /// ```
-    /// use lib_rapid::compsci::general::Brackets;
-    /// 
-    /// let s = "([{Some text won't throw it off}])";
-    /// 
-    /// assert_eq!(Ok(true), s.validate_brackets());
-    /// ```
-    /// ```
-    /// use lib_rapid::compsci::general::Brackets;
-    /// 
-    /// let s = String::from("([{}])");
-    /// 
-    /// assert_eq!(Ok(true), s.validate_brackets());
-    /// ```
-    /// ```
-    /// use lib_rapid::compsci::general::Brackets;
-    /// 
-    /// let s = "([{}]";
-    /// 
-    /// assert_eq!(Err(5), s.validate_brackets());
-    /// ```
-    /// ```
-    /// use lib_rapid::compsci::general::Brackets;
-    /// 
-    /// let s = "([{Some text won't throw it off}]))";
-    /// 
-    /// assert_eq!(Err(34), s.validate_brackets());
-    /// ```
-    /// ```
-    /// use lib_rapid::compsci::general::Brackets;
-    /// 
-    /// let s = "([{]}))";
-    /// 
-    /// assert_eq!(Err(3), s.validate_brackets());
-    /// ```
-    fn validate_brackets(&self) -> Result<bool, usize>;
-}
 /// Simple methods for characters.
 pub trait CharTools {
     /// Determines whether a character is a opening bracket.
@@ -344,7 +239,7 @@ pub trait FloatMagic {
     /// 
     /// assert_eq!(2570638229164439, 3.1415927_f64.raw_mantissa());
     /// ```
-    #[must_use]
+    #[must_use = "This returns the result of the operation, without modifying the original."]
     fn raw_mantissa(&self) -> Self::Mantissa;
     /// Get the raw binary exponent data.
     /// # Returns
@@ -356,7 +251,7 @@ pub trait FloatMagic {
     /// assert_eq!(1024, 3.1415927_f64.raw_exponent());
     /// assert_eq!(1, 1.1754943508e-38f32.raw_exponent());
     /// ```
-    #[must_use]
+    #[must_use = "This returns the result of the operation, without modifying the original."]
     fn raw_exponent(&self) -> Self::Exponent;
     /// Get the actual mantissa part.
     /// # Returns
@@ -368,7 +263,7 @@ pub trait FloatMagic {
     /// assert_eq!(1.57079635, 3.1415927_f64.real_mantissa());
     /// assert_eq!(1.0, 1.1754943508e-38f32.real_mantissa());
     /// ```
-    #[must_use]
+    #[must_use = "This returns the result of the operation, without modifying the original."]
     fn real_mantissa(&self) -> Self;
     /// Get the actual exponent part.
     /// # Returns
@@ -381,7 +276,7 @@ pub trait FloatMagic {
     /// assert_eq!(-126, 1.1754943508e-38f32.real_exponent());
     /// assert_eq!(-1022, 2.2250738585072014e-308.real_exponent()); // Minimal f64 value.
     /// ```
-    #[must_use]
+    #[must_use = "This returns the result of the operation, without modifying the original."]
     fn real_exponent(&self) -> Self::RealExponent;
     /// Decompose a floating point number into it's core components.
     /// # Returns
@@ -395,7 +290,7 @@ pub trait FloatMagic {
     /// 
     /// assert_eq!((0, 1024, 2570638229164439), 3.1415927_f64.raw_decompose());
     /// ```
-    #[must_use]
+    #[must_use = "This returns the result of the operation, without modifying the original."]
     fn raw_decompose(&self) -> (u8, Self::Exponent, Self::Mantissa);
     /// Decompose a floating point number into it's core components.
     /// # Returns
@@ -410,7 +305,7 @@ pub trait FloatMagic {
     /// assert_eq!((0, 1, 1.5707964), 3.1415927_f32.real_decompose());
     /// assert_eq!((0, 1, 1.57079635), 3.1415927_f64.real_decompose());
     /// ```
-    #[must_use]
+    #[must_use = "This returns the result of the operation, without modifying the original."]
     fn real_decompose(&self) -> (u8, Self::RealExponent, Self);
     /// Compose a floating point value out of the raw parts.
     /// # Returns
@@ -418,8 +313,7 @@ pub trait FloatMagic {
     /// # Panics
     /// Panics if one of the parts is out of range for `Self`.
     /// # Examples
-    /// ```    fn raw_exponent(&self) -> Self::Exponent {
-
+    /// ```
     /// use lib_rapid::compsci::general::FloatMagic;
     /// 
     /// let my_float: f32 = 3.1415927;
@@ -427,37 +321,34 @@ pub trait FloatMagic {
     /// let exponent      = my_float.raw_exponent();
     /// let mantissa      = my_float.raw_mantissa();
     /// 
-    /// let my_double       = my_float as f64;
-    /// let sign_1          = my_double.is_sign_negative() as u8;
-    /// let exponent_1      = my_double.raw_exponent();
-    /// let mantissa_1      = my_double.raw_mantissa();
+    /// let my_double: f64 = 3.141592653589793;
+    /// let sign_1         = my_double.is_sign_negative() as u8;
+    /// let exponent_1     = my_double.raw_exponent();
+    /// let mantissa_1     = my_double.raw_mantissa();
     /// 
     /// assert_eq!(3.1415927, f32::raw_compose(sign, exponent, mantissa));
-    /// assert_eq!(3.1415927410125732, f64::raw_compose(sign, exponent_1, mantissa_1));
+    /// assert_eq!(3.141592653589793, f64::raw_compose(sign_1, exponent_1, mantissa_1));
     /// ```
     #[must_use]
     fn raw_compose(sign: u8, exponent: Self::Exponent, mantissa: Self::Mantissa) -> Self;
 }
 
 impl CharTools for char {
+    #[inline]
     fn is_open_bracket(&self) -> bool {
         match self {
-            '(' => { true }
-            '{' => { true }
-            '[' => { true }
-            _   => { false }
+            '(' | '{' | '[' => { true }
+            _               => { false }
         }
     }
-
+    #[inline]
     fn is_closed_bracket(&self) -> bool {
         match self {
-            ')' => { true }
-            '}' => { true }
-            ']' => { true }
-            _   => { false }
+            ')' | '}' | ']' => { true }
+            _               => { false }
         }
     }
-
+    #[inline]
     fn get_closing_bracket(&self) -> Option<char> {
         match self {
             '(' => { Some(')') }
@@ -466,7 +357,7 @@ impl CharTools for char {
             _   => { None }
         }
     }
-
+    #[inline]
     fn get_opening_bracket(&self) -> Option<char> {
         match self {
             ')' => { Some('(') }
@@ -523,8 +414,8 @@ impl FloatMagic for f32 {
 }
 
 impl FloatMagic for f64 {
-    type Mantissa = u64;
-    type Exponent = u16;
+    type Mantissa     = u64;
+    type Exponent     = u16;
     type RealExponent = i32;
 
     #[inline]
@@ -560,45 +451,11 @@ impl FloatMagic for f64 {
     fn raw_compose(sign: u8, exponent: Self::Exponent, mantissa: Self::Mantissa) -> Self {
         if sign     > 1                { panic!("A sign bigger than 1 is not allowed."); }
         if exponent > 2047             { panic!("An exponent bigger than 2047 is not allowed."); }
-        if mantissa > 4503599627370000 { panic!("A mantissa bigger than 8388607 is not allowed."); }
+        if mantissa > 4503599627370000 { panic!("A mantissa bigger than 4503599627370000 is not allowed."); }
 
         unsafe { transmute((mantissa |
-                          ((exponent as u64) << 23)) |
-                           (sign     as u64) << 31) }
-    }
-}
-
-impl Brackets for &str {
-    fn validate_brackets(&self) -> Result<bool, usize> {
-        backend_val_brackets(self)
-    }
-}
-
-impl Brackets for String {
-    fn validate_brackets(&self) -> Result<bool, usize> {
-        backend_val_brackets(self)
-    }
-}
-
-impl StringIndex for String {
-    #[inline]
-    fn char_at(&self, index: usize) -> Option<char> {
-        self.chars().nth(index)
-    }
-    #[inline]
-    fn byte_at(&self, index: usize) -> u8 {
-        self.as_bytes()[index]
-    }
-}
-
-impl StringIndex for &str {
-    #[inline]
-    fn char_at(&self, index: usize) -> Option<char> {
-        self.chars().nth(index)
-    }
-    #[inline]
-    fn byte_at(&self, index: usize) -> u8 {
-        self.as_bytes()[index]
+                          ((exponent as u64) << 52)) |
+                           (sign     as u64) << 63) }
     }
 }
 
@@ -635,16 +492,8 @@ impl<T: BitXorAssign +
     where
     <T as TryFrom<U>>::Error: Debug {
     fn xor_with(&self, other: &[U]) -> Vec<T> {
-        if size_of_val(self) != size_of_val(other)
-        { panic!("{}", BITWISE_ERR1); }
-
-        let t_size: usize = size_of::<T>();
-        let u_size: usize = size_of::<U>();
-        if t_size < u_size
-        { panic!("{}", BITWISE_ERR2); }
-        
+        let multiplier: usize  = bitwise_preproc::<T, U>(self, other);
         let mut _res:   Vec<T> = self.to_vec();
-        let multiplier: usize  = t_size / u_size;
 
         for (index, slice) in other.chunks(multiplier).enumerate() {
             _res[index] ^= bitwise_from(slice);
@@ -654,16 +503,9 @@ impl<T: BitXorAssign +
     }
 
     fn or_with(&self, other: &[U]) -> Vec<T> {
-        if size_of_val(self) != size_of_val(other)
-        { panic!("{}", BITWISE_ERR1); }
-
-        let t_size: usize = size_of::<T>();
-        let u_size: usize = size_of::<U>();
-        if t_size < u_size
-        { panic!("{}", BITWISE_ERR2); }
-
+        let multiplier: usize  = bitwise_preproc::<T, U>(self, other);
         let mut _res:   Vec<T> = self.to_vec();
-        let multiplier: usize  = t_size / u_size;
+
         for (index, slice) in other.chunks(multiplier).enumerate() {
             _res[index] |= bitwise_from(slice);
         }
@@ -672,16 +514,8 @@ impl<T: BitXorAssign +
     }
 
     fn and_with(&self, other: &[U]) -> Vec<T> {
-        if size_of_val(self) != size_of_val(other)
-        { panic!("{}", BITWISE_ERR1); }
-
-        let t_size: usize = size_of::<T>();
-        let u_size: usize = size_of::<U>();
-        if t_size < u_size
-        { panic!("{}", BITWISE_ERR2); }
-        
+        let multiplier: usize  = bitwise_preproc::<T, U>(self, other);
         let mut _res:   Vec<T> = self.to_vec();
-        let multiplier: usize  = t_size / u_size;
 
         for (index, slice) in other.chunks(multiplier).enumerate() {
             _res[index] &= bitwise_from(slice);
@@ -740,24 +574,14 @@ fn bitwise_from<T: From<u8> +
     end_prod
 }
 
-#[must_use]
-fn backend_val_brackets(s: &str) -> Result<bool, usize> {
-    let mut res_vec: Vec<char> = Vec::with_capacity(s.len());
-    let mut i: usize = 0;
-    for c in s.chars() {
-        match c {
-            '[' => { res_vec.push(c); },
-            '{' => { res_vec.push(c); },
-            '(' => { res_vec.push(c); },
-            ']' if res_vec.pop() != Some('[') => { return Err(i); }
-            '}' if res_vec.pop() != Some('{') => { return Err(i); }
-            ')' if res_vec.pop() != Some('(') => { return Err(i); }
-            _   => { }
-        }
-        i.inc();
-    }
+#[inline(always)]
+fn bitwise_preproc<T,U>(a: &[T], b: &[U]) -> usize {
+    if size_of_val(a) != size_of_val(b)
+    { panic!("{}", BITWISE_ERR1); }
 
-    if !res_vec.is_empty()
-    { return Err(i); }
-    Ok(true)
+    let t_size: usize = size_of::<T>();
+    let u_size: usize = size_of::<U>();
+    if t_size < u_size
+    { panic!("{}", BITWISE_ERR2); }
+    t_size / u_size
 }
\ No newline at end of file
