@@ -1,9 +1,7 @@
 //! Basic logging functionality.
 use chrono::{Utc, SecondsFormat};
-use std::io::prelude::*;
-use std::fs::OpenOptions;
-
-use crate::math::general::Increment;
+use std::{io::prelude::*, fs::OpenOptions};
+use crate::math::general::NumTools;
 
 /// The structure for the RapidLogger.
 pub struct Logger {
